/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*
 */

#include "nsISupports.idl"

interface nsIURI;
interface nsIPrincipal;

/**
 * The nsILoadURIDelegate interface.
 * Used for delegating URI loads to GeckoView's application, e.g., Custom Tabs
 * or Progressive Web Apps.
 */
[scriptable, uuid(78e42d37-a34c-4d96-b901-25385669aba4)]
interface nsILoadURIDelegate : nsISupports
{
  /**
   * Delegates the URI load.
   *
   * @param aURI The URI to load.
   * @param aWhere See possible values described in nsIBrowserDOMWindow.
   * @param aFlags Flags which control the behavior of the load.
   * @param aTriggeringPrincipal The principal that triggered the load of aURI.
  */
  void
  loadURI(in nsIURI aURI, in short aWhere, in long aFlags,
          in nsIPrincipal aTriggeringPrincipal);
};
