// Copyright (C) 2015 the V8 project authors. All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.
/*---
es6id: 23.3.3.2
esid: sec-weakmap.prototype.delete
description: >
  Throws TypeError if `this` doesn't have a [[WeakMapData]] internal slot.
info: >
  WeakMap.prototype.delete ( value )

  ...
  3. If M does not have a [[WeakMapData]] internal slot, throw a TypeError
  exception.
  ...
---*/

assert.throws(TypeError, function() {
  WeakMap.prototype.delete.call([], {});
});

assert.throws(TypeError, function() {
  var map = new WeakMap();
  map.delete.call([], {});
});

reportCompare(0, 0);
